@@ -1,6 +1,6 @@
 # The root directory for the `cfl` package.
 # This folder contains all the functional code necessary to run `cfl`.
 
-__version__ = '1.0.0' # version <1.0 indicates 'pre-release' 
+__version__ = '1.0.2' 
 __author__= 'Jenna Kahn and Iman Wahle'
-__credits__= 'California Institute of Technology'
+__credits__= '2021, California Institute of Technology'
