@@ -1,19 +1,21 @@
 """
-a helper file used by `kmeans.py` to find P(Y=y|X=Xclass),
+a helper file used before effect-side clustering to find P(Y=y|X=Xclass),
 the conditional probability of each y value, given each X-macrovariable.
 
 Contains functions for clustering categorical and continuous 1-D Ys
 (not tested on higher dimensional Y)
 """
+
 import numpy as np
 from tqdm import tqdm
+from sklearn.metrics.pairwise import euclidean_distances
+from joblib import Parallel, delayed
+
+
 from cfl.util.find_xlbl_locations import rows_where_each_x_class_occurs
 from cfl.util.data_processing import one_hot_decode
-from sklearn.metrics.pairwise import euclidean_distances
 
-
-
-def sample_Y_dist(Y_type, dataset, x_lbls):
+def sample_Y_dist(Y_type, dataset, x_lbls, precompute_distances=True):
     #TODO: is name good? I think it's decent
     """
     Finds (a proxy of) P(Y=y | Xclass) for all Y=y
@@ -34,15 +36,16 @@ def sample_Y_dist(Y_type, dataset, x_lbls):
     """
     Y = dataset.get_Y()
     if Y_type == 'continuous':
-        y_probs = _continuous_Y(Y, x_lbls)
+        y_probs = _continuous_Y(Y, x_lbls, precompute_distances)
     elif Y_type == 'categorical':
-        y_probs = _categorical_Y(Y, x_lbls)
+        y_probs = _categorical_Y(Y, x_lbls, precompute_distances)
     else: 
         raise TypeError('Invalid Y-type')
     return y_probs
 
 
-def _categorical_Y(Y_data, x_lbls):
+def _categorical_Y(Y_data, x_lbls, precompute_distances=True):
+    # TODO: use precompute_distances
     """
     Estimates the conditional probability density P(Y=y|X=xClass) for
     categorical data, where 'y' is an observation in Y_data and xClass is a
@@ -55,6 +58,9 @@ def _categorical_Y(Y_data, x_lbls):
         x_lbls (np.ndarray): a 1D array (same
             length/aligned with Y_data) of the CFL labels predicted for the x
             (cause) data
+        precompute_distances (boolean): when True, distances between all 
+            samples will be precomputed. This will significantly speed up
+            this function, but uses considerable space for larger datasets.
 
     Returns: 
         np.ndarray: an array with a row for each observation
@@ -99,7 +105,7 @@ def _categorical_Y(Y_data, x_lbls):
     return cond_Y_prob
 
 
-def _continuous_Y(Y_data, x_lbls):
+def _continuous_Y(Y_data, x_lbls, precompute_distances=True):
     """
     Estimates the conditional probability density `P(Y=y|X=xClass)` for every y
     (observation in Y_data) and xClass (macrovariable constructed from X_data,
@@ -128,6 +134,9 @@ def _continuous_Y(Y_data, x_lbls):
         x_lbls (np.ndarray): a 1D array (same
             length/aligned with `Y_data`) of the CFL labels predicted for the 
             `X` (cause) data
+        precompute_distances (boolean): when True, distances between all 
+            samples will be precomputed. This will significantly speed up
+            this function, but uses considerable space for larger datasets.
 
     Returns: 
         np.ndarray: a 2D array with a row for each observation in Y_data
@@ -136,7 +145,8 @@ def _continuous_Y(Y_data, x_lbls):
             y value, given that the x is a member of the corresponding class of 
             that column.
 
-    Note: Why is `P(y|x_Class)` calculated, instead of `P(y|x)` for each
+    Note: 
+        Why is `P(y|x_Class)` calculated, instead of `P(y|x)` for each
         individual `x`? The clusters of `x` created immediately prior to this
         step are observational classes of `X` (see "Causal Feature Learning: An
         Overview" by Eberhardt, Chalupka, Pierona 2017). Observational classes
@@ -155,44 +165,113 @@ def _continuous_Y(Y_data, x_lbls):
     # format x_lbls that come in as shape (n_samples,1) to be of shape 
     # (n_samples,)
     x_lbls = np.squeeze(x_lbls)
-
-    # precompute distance matrices
-    dist_matrix = euclidean_distances(Y_data, Y_data)
-
-    # separate distances by X class
-    # xclass_dist_matrix is a 3D array of distances where:
-    #   - axis 0 corresponds to X classes
-    #   - axis 1 corresponds to Y_data samples
-    #   - axis 2 corresponds to nearest neighbors, sorted
-    k_neighbors = 4
-    n_x_classes = len(np.unique(x_lbls))
-    xclass_dist_matrix = -1 * np.ones((n_x_classes, Y_data.shape[0], 
-                                       k_neighbors))
-    for xi in np.unique(x_lbls):
-        # take first through fifth closest distances (offset by one to 
-        # leave out distance to self)
-        nn = np.sort(dist_matrix[:,x_lbls==xi],axis=1)[:,1:k_neighbors+1]
-        # if we have less than k_neighbors neighbors, pad with -1
-        padded_nn = np.pad(nn, ((0,0),(0,k_neighbors-nn.shape[1])), 'constant', 
-                           constant_values=-1) 
-        xclass_dist_matrix[xi,:,:] = padded_nn
+    k_neighbors = 4 #4 nearest neighbors is hard-coded in because that is what the El Nino paper, which introduced this method, used
+    class_labels = np.unique(x_lbls)
+    n_x_classes = len(class_labels)
+    
 
     # now we can compute nearest neighbors averages for each 
     # Y_data point and X class combination
     cond_Y_prob = np.zeros((Y_data.shape[0], n_x_classes))
-    # vectorize this
-    for y_id in tqdm(range(Y_data.shape[0])):
-        # compute average distance to nearest neighbors per class
-        for xi in range(xclass_dist_matrix.shape[0]):
-            # do not include -1 entries in mean
-            valid = xclass_dist_matrix[xi,y_id,:] >= 0
-            cond_Y_prob[y_id,xi] = np.mean(xclass_dist_matrix[xi,y_id,valid])
 
-        # normalize so that sum of distances is 1
-        cond_Y_prob[y_id,:] /=np.sum(cond_Y_prob[y_id,:])
+    # precompute distance matrices
+    if precompute_distances:
+        dist_matrix = euclidean_distances(Y_data, Y_data)
+
+        # separate distances by X class
+        # xclass_dist_matrix is a 3D array of distances where:
+        #   - axis 0 corresponds to X classes
+        #   - axis 1 corresponds to Y_data samples
+        #   - axis 2 corresponds to nearest neighbors, sorted
+        xclass_dist_matrix = -1 * np.ones((n_x_classes, Y_data.shape[0], 
+                                        k_neighbors))
+                                        
+        for xi,xlbl in enumerate(class_labels):
+            # take first through fifth closest distances (offset by one to 
+            # leave out distance to self)
+            nn = np.sort(dist_matrix[:,x_lbls==xlbl],axis=1)[:,1:k_neighbors+1]
+            # if we have less than k_neighbors neighbors, pad with -1
+            padded_nn = np.pad(nn, ((0,0),(0,k_neighbors-nn.shape[1])), 
+                'constant', constant_values=-1) 
+            xclass_dist_matrix[xi,:,:] = padded_nn
+    
+        for y_id in tqdm(range(Y_data.shape[0])):
+            # compute average distance to nearest neighbors per class
+            for xi,xlbl in enumerate(class_labels):
+
+                # do not include -1 entries in mean
+                valid = xclass_dist_matrix[xi,y_id,:] >= 0
+                cond_Y_prob[y_id,xi] = \
+                    np.mean(xclass_dist_matrix[xi,y_id,valid])
+
+            # normalize so that sum of distances is 1
+            # note: not only is it unecessary to normalize when we only have once
+            # class (as the only element in the row equals the sum of the row), 
+            # but it can also cause an error when we have duplicate points. For
+            # example, if we have five duplicate points and they are all in the
+            # same (and only) class, the four nearest neighbors to one of these
+            # points will all have distance 0, so the sum of the row will be 0 -->
+            # divide by zero error. We will avoid dividing in the case all together
+            # since it's not needed anyways.
+            if n_x_classes > 1:
+                cond_Y_prob[y_id,:] /= np.sum(cond_Y_prob[y_id,:])
+    
+    else:
+        def parallel_job1(y_id, xi, xlbl):
+            # compute distances from sample y_id to all other samples in 
+            # class xi
+            y_id_dists = np.squeeze(euclidean_distances(np.expand_dims(Y_data[y_id,:],0), Y_data[x_lbls==xlbl,:]))
+            
+            # take average of closest neighbors in class xi
+            cond_Y_prob[y_id,xi] = \
+                np.mean(np.sort(y_id_dists)[1:k_neighbors+1])
+            
+        loop1 = [delayed(parallel_job1)(y_id, xi, xlbl) for xi,xlbl in enumerate(class_labels) for y_id in tqdm(range(Y_data.shape[0]))]
+        Parallel(n_jobs=-1, prefer='threads')(loop1)
+
+        # normalize so that sum of distances is 1 (see note above)
+        if n_x_classes > 1:
+            def parallel_job2(y_id):
+                cond_Y_prob[y_id,:] /= np.sum(cond_Y_prob[y_id,:])
+            loop2 = [delayed(parallel_job2)(y_id) for y_id in range(Y_data.shape[0])]
+            Parallel(n_jobs=-1, prefer='threads')(loop2)
+
 
     return cond_Y_prob
 
+
+
+# def _continuous_Y_parallelized(Y_data, x_lbls, precompute_distances=True):
+
+#     assert Y_data.shape[0] == x_lbls.shape[0], "The Y data and x_lbls arrays \
+#         passed through continuous_Y should have the same length. Actual \
+#         shapes: {},{}".format(Y_data.shape[0],x_lbls.shape[0])
+
+#     # format x_lbls that come in as shape (n_samples,1) to be of shape 
+#     # (n_samples,)
+#     x_lbls = np.squeeze(x_lbls)
+#     k_neighbors = 4
+#     n_x_classes = len(np.unique(x_lbls))
+
+#     # now we can compute nearest neighbors averages for each 
+#     # Y_data point and X class combination
+#     cond_Y_prob = np.zeros((Y_data.shape[0], n_x_classes))
+
+#     def what_to_parallelize(y_id, xi):
+#         # compute distances from sample y_id to all other samples in 
+#         # class xi
+#         y_id_dists = np.squeeze(euclidean_distances(np.expand_dims(Y_data[y_id,:],0), Y_data[x_lbls==xi,:]))
+        
+#         # take average of closest neighbors in class xi
+#         cond_Y_prob[y_id,xi] = \
+#             np.mean(np.sort(y_id_dists)[1:k_neighbors+1])
+        
+#     loop = [delayed(what_to_parallelize)(0, xi) for xi in range(n_x_classes) for y_id in range(Y_data.shape[0])]
+#     Parallel(n_jobs=-1, prefer='threads')(loop)
+#     return cond_Y_prob
+
+
+
     # TODO: this version hasn't been implemented for the categorical case yet
 
 # def _avg_nearest_neighbors_dist(y, other_Ys, y_in_otherYs, k_neighbors=4):
