@@ -1,12 +1,13 @@
 from abc import abstractmethod
 import pickle #for saving code
 
+import numpy as np
+from sklearn.cluster import DBSCAN
+
 from cfl.block import Block
 from cfl.dataset import Dataset
-import numpy as np
 from cfl.cluster_methods.Y_given_Xmacro import sample_Y_dist # calculate 
                                                              # P(Y|Xmacro)
-from sklearn.cluster import DBSCAN
 
 #TODO: next step: add very clear documentation about how to add new module. 
 # Include:
@@ -69,21 +70,22 @@ class Clusterer(Block):
 
     def __init__(self, data_info, params):
         """
-        initialize Clusterer object
+        Initialize Clusterer object
 
         Parameters
             data_info (dict): 
             params (dict) : a dictionary of relevant hyperparameters for
-                clustering. For a base clusterer object, these should be two 
-                already created clusterers (one for clustering the x data and 
-                one for the y. The parameters between the two can be the same). 
-                The clusterer object needs to adhere to the Scikit learn 
-                BaseEstimator (https://scikit-learn.org/stable/modules/generated/sklearn.base.BaseEstimator.html)
-                and ClusterMixin Interfaces (https://scikit-learn.org/stable/modules/generated/sklearn.base.ClusterMixin.html)
-                ^ (this means they need to have the method 
-                fit_predict(X, y=None) and assign the results as 
-                self.labels_ #TODO: make this docstring and the requirements 
-                better.
+                clustering. This dictionary should contain the keys `x_model` and
+                `y_model`. The value of each of these should be an already created
+                clustering object (the first for clustering the `X` data and the
+                second for clustering the `Y` data). `None` can be passed as the
+                value of `y_model` to indicate only clustering on the cause data. 
+
+                The clusterer objects need to adhere to the Scikit learn 
+                `BaseEstimator` (https://scikit-learn.org/stable/modules/generated/sklearn.base.BaseEstimator.html)
+                and `ClusterMixin` interfaces (https://scikit-learn.org/stable/modules/generated/sklearn.base.ClusterMixin.html)
+                This means they need to have the method `fit_predict(X, y=None)` and assign the results as 
+                `self.labels_`.
 
         Return
             None
@@ -123,6 +125,7 @@ class Clusterer(Block):
 
         default_params =  {'x_model' : DBSCAN(),
                            'y_model' : DBSCAN(),
+                           'precompute_distances' : True,
                           }
         return default_params
 
@@ -158,22 +161,37 @@ class Clusterer(Block):
         self.xmodel.fit(pyx)
         x_lbls = self.xmodel.labels_
 
-        # sample P(Y|Xclass)
-        y_probs = sample_Y_dist(self.Y_type, dataset, x_lbls)
+        # if we are also clustering effect data 
+        if self.ymodel is not None: 
+            # sample P(Y|Xclass)
+            y_probs = sample_Y_dist(self.Y_type, dataset, x_lbls, 
+                precompute_distances=self.params['precompute_distances'])
 
-        # do y clustering
-        self.ymodel.fit(y_probs)
-        y_lbls = self.ymodel.labels_
+            # do y clustering
+            self.ymodel.fit(y_probs)
+            y_lbls = self.ymodel.labels_
+
+            self.trained = True
+            results_dict = {'x_lbls'  : x_lbls,
+                            'y_lbls'  : y_lbls,
+                            'y_probs' : y_probs}
+        else: 
+            results_dict = {'x_lbls'  : x_lbls}
 
-        self.trained = True
-        results_dict = {'x_lbls' : x_lbls,
-                        'y_lbls' : y_lbls}
         return results_dict
 
+    # TODO: the name 'predict' is still a lie because 'fit_predict' is not the
+    # same as predict
     def predict(self, dataset, prev_results):
         """  
         Assign new datapoints to clusters found in training.
 
+        NOTE: 
+            TODO: fit_predict is different than predict, so this code is WRONG  
+        however, kmeans is the only clustering function in sklearn that has 
+        a predict function defined so we're doing this for now
+
+
         Arguments:
             dataset (Dataset): Dataset object containing X, Y and pyx data to 
                                assign partition labels to 
@@ -197,14 +215,19 @@ class Clusterer(Block):
         pyx = prev_results['pyx']
 
         x_lbls = self.xmodel.fit_predict(pyx)
-        # NOTE: fit_predict is different than predict, so this is wrong 
-        # however, kmeans is the only clustering function in sklearn that has 
-        # a predict function defined so we're doing this for now
-        y_probs = sample_Y_dist(self.Y_type, dataset, x_lbls)
-        y_lbls = self.ymodel.fit_predict(y_probs)
 
-        results_dict = {'x_lbls' : x_lbls,
-                        'y_lbls' : y_lbls}
+        # if we are also clustering effect data 
+        if self.ymodel is not None: 
+            # sample P(Y|Xclass)
+            y_probs = sample_Y_dist(self.Y_type, dataset, x_lbls)
+
+            # do y clustering
+            y_lbls = self.ymodel.fit_predict(y_probs)
+
+            results_dict = {'x_lbls' : x_lbls,
+                            'y_lbls' : y_lbls}
+        else: 
+            results_dict = {'x_lbls' : x_lbls}
         return results_dict
 
 
