@@ -24,13 +24,9 @@ class CondExpCNN(CondExpBase):
                 data that will be passed in. Should contain 'X_dims',
                 'Y_dims', and 'Y_type' as keys.
             params (dict) : dictionary containing parameters for the model.
-            model (str) : name of the model so that the model type can be
-                recovered from saved parameters.
         Returns: 
             None
         '''
-
-        self.name='CondExpCNN'
         super().__init__(data_info, params) #Main init stuff happens in block.py
 
 
@@ -38,9 +34,9 @@ class CondExpCNN(CondExpBase):
         '''         
         Define the neural network based on specifications in self.params.
 
-        This creates a convolutional neural net with the structure
+        **This creates a convolutional neural net with the structure
         (Conv2D layer, MaxPooling2D layer) * n, Flatten layer, Dense layer(s), 
-        Output layer
+        Output layer** 
 
         The number of Conv2d/Maxpooling layers is determined by the length of 
         the filter/kernel_size/pool_size parameter lists given in the params 
@@ -124,7 +120,7 @@ class CondExpCNN(CondExpBase):
                           'batch_size'  : 32,
                           'n_epochs'    : 20,
                           'optimizer'   : 'adam',
-                            'opt_config'  : {},
+                          'opt_config'  : {},
                             'verbose'     : 1,
                             'weights_path': None,
                             'loss'        : 'mean_squared_error',
