@@ -25,12 +25,9 @@ class CondExpKC(CondExpBase):
                 data that will be passed in. Should contain 'X_dims',
                 'Y_dims', and 'Y_type' as keys.
             params (dict) : dictionary containing parameters for the model.
-            model (str) : name of the model so that the model type can be
-                recovered from saved parameters.
         Returns: 
             None
         '''
-        self.name = 'CondExpKC'
         super().__init__(data_info, params)
 
     def _get_default_params(self):
