@@ -23,12 +23,9 @@ class CondExpMod(CondExpBase):
                 data that will be passed in. Should contain 'X_dims', 
                 'Y_dims', and 'Y_type' as keys.
             params (dict) : dictionary containing parameters for the model.
-            model (str) : name of the model so that the model type can be
-                recovered from saved parameters.
         Returns: 
             None
         '''
-        self.name = 'CondExpMod'
         super().__init__(data_info=data_info, params=params)
 
     def _get_default_params(self):
