@@ -6,8 +6,7 @@ import numpy as np
 from cfl.dataset import Dataset
 from cfl.block import Block
 import cfl.density_estimation_methods as cdem
-# import cfl.cluster_methods as ccm
-from cfl.cluster_methods import clusterer
+import cfl.cluster_methods as ccm
 from cfl import intervention_rec
 
 '''
@@ -26,13 +25,12 @@ Methods in Experiment Class:
 
 '''
 # TODO: this is a placeholder until we have a block registration system.
-# NOTE: the name in the registry has to match the self.name in each block's __init__ 
-BLOCK_KEY = {   'CondExpVB'     : cdem.CondExpVB, 
-                'CondExpKC'     : cdem.CondExpKC,
-                'CondExpCNN'    : cdem.CondExpCNN,
-                'CondExpCNN3D'  : cdem.CondExpCNN3D,
-                'CondExpMod'    : cdem.CondExpMod,
-                'Clusterer'     : clusterer.Clusterer } #TODO: maybe change this so that instead of 
+# NOTE: the keys of this dictionary are passed as part of the 'block_names'
+# list. They are different than the names of the attributes in each block's
+# self.name attribute
+BLOCK_KEY = {   'CDE'             : cdem.CDE,
+                'CauseClusterer'  : ccm.CauseClusterer,
+                'EffectClusterer' : ccm.EffectClusterer} #TODO: maybe change this so that instead of 
                                                             # calling clusterer, 'Kmeans', 'DBSCAN' and 'SNN' are registered as cluster methods  
 
 class Experiment():
@@ -63,7 +61,9 @@ class Experiment():
             block_params (list of dicts) : list of dicts specifying parameters for each block specified
                            in block_names. Default is None. 
             blocks (list of Blocks): list of block objects. Default is None.
-            results_path (str) : path to directory to save this experiment to. Default is `''`. 
+            verbose (int): Amount of output to print. Possible values are 0, 1, 2. Default is 1.
+            results_path (str) : path to directory to save this experiment to.
+                Default is `''`.
 
         Note: There are three ways to specify Blocks: 
                 1) specify `past_exp_path`
@@ -71,29 +71,44 @@ class Experiment():
                 3) specify `blocks`. 
               Do not specify all four of these parameters. 
         '''
+        self.verbose = verbose
 
-        # # check data input types
-        # assert isinstance(X_train, np.ndarray), 'X should be of type np.ndarray'
-        # assert isinstance(Y_train, np.ndarray), 'Y should be of type np.ndarray' #TODO: remove (duplicate of work done by Dataset class)
-
+        # OPTION 1 for Experiment initialization: load from path. 
         # if loading from past experiment, make sure no other block
-        # specifications are provided ...
+        # specifications are provided
         if past_exp_path is not None:
             assert (block_names is None), 'block_names should not be specified.'
             assert (block_params is None), 'block_params should not be specified.'
             assert (blocks is None), 'blocks should not be specified.'
 
-        # otherwise, make sure block names and params are both provided, and that 
-        # blocks is left unpopulated ...
-        if (block_names is not None) or (block_params is not None):
+            # load in block names and params
+            print(f'Loading in Experiment from {past_exp_path}')
+            block_names, block_params = self.__load_params(os.path.join(past_exp_path, 'params'))
+
+
+        # OPTION 2 for Experiment initialization: create blocks from strings. 
+        # make sure block names and params are both provided, and that 
+        # blocks is left unpopulated 
+        elif (block_names is not None) or (block_params is not None):
             assert (block_names is not None), 'block_names should be specified.'
             assert (block_params is not None), 'block_params should be specified.'
             assert (blocks is None), 'blocks should not be specified.'
+            self.is_trained = False
+            # add verbosity to params that don't specify
+            # removed because was causing problems with some sklearn models that don't have a verbose param
+            # block_params = self.__propagate_verbosity(self.verbose, block_params)
         
-        # otherwise, make sure that only blocks is provided.
-        if blocks is not None:
+        #OPTION 3 for Experiment initialization: blocks pre-created. 
+        # make sure that only blocks is provided.
+        elif blocks is not None:
             assert (block_names is None), 'block_names should not be specified.'
             assert (block_params is None), 'block_params should not be specified.'
+            
+            for block in self.blocks:
+                assert isinstance(block, Block), \
+                    'A specified block is not of type Block.'
+
+            self.is_trained = False
 
         # make sure one of the three Experiment definitions is supplied
         assert (past_exp_path is not None) or \
@@ -104,7 +119,6 @@ class Experiment():
         # Note: explicitly stating one dataset for training as an Experiment
         # attribute enforces the definition that an Experiment is a unique 
         # configuration of a trained CFL.
-        self.is_trained = False
         self.data_info = data_info
         self.datasets = {}
         self.dataset_train = self.add_dataset(X=X_train, Y=Y_train, \
@@ -112,25 +126,17 @@ class Experiment():
                                               Xraw=X_train_raw, \
                                               Yraw=Y_train_raw)
         self.datasets[self.dataset_train.get_name()] = self.dataset_train
-
-        # add verbosity to params that don't specify
-        self.verbose = verbose
-        if block_params is not None:
-            block_params = self.__propagate_verbosity(self.verbose, block_params)
         
         # build experiment directory
         self.save_path = self.__make_exp_dir(results_path)
 
-        # load in params from past experiment if provided
-        if past_exp_path is not None:
-            block_names, block_params = self.__load_params(os.path.join(past_exp_path, 'params'))
-
-        # build blocks from names and params if blocks not provided
+        # build blocks from names and params if blocks not provided (ie in Options 1
+        # or 2)
         if blocks is None:
             blocks = []
             for bn,bp in zip(block_names, block_params): # data_info
                 blocks.append(self.__build_block(bn,bp))
-        
+
         # load in trained block info if past experiment provided
         if past_exp_path is not None:
             for block in blocks:
@@ -138,10 +144,6 @@ class Experiment():
                 block.load_block(fn)
             self.is_trained = True
 
-        self.blocks = blocks
-        for block in self.blocks:
-            assert isinstance(block, Block), \
-                'A specified block is not of type Block.'
 
         # TODO: check that interfaces match
         # TODO: assert in the function itself so we can give more info
@@ -149,6 +151,9 @@ class Experiment():
         # assert self.check_blocks_compatibility(), 'Specified blocks are incompatible'
         
         # save configuration parameters for each block
+        self.blocks = blocks
+        self.block_names = block_names
+        self.block_params = block_params
         self.__save_params()
 
     def train(self, dataset=None, prev_results=None):
@@ -256,8 +261,243 @@ class Experiment():
         dataset.set_cfl_results(all_results)
         return all_results        
 
+    def add_dataset(self, X, Y, dataset_name, Xraw=None, Yraw=None):
+        ''' Add a new dataset to be tracked by this Experiment. 
+            
+            Arguments: 
+                X (np.array) : X data of shape (n_samples, n_x_features) associated with 
+                    this Dataset. 
+                Y (np.array) : Y data of shape (n_samples, n_y_features) associated with
+                    this Dataset. 
+                dataset_name (str) : name associated with this Dataset. This will be
+                               the name used to retrieve a dataset using the
+                               `Experiment.get_dataset()` method. 
+                Xraw (np.ndarray) : (Optional) raw form of X before preprocessing to remain associated with X for visualization. Defaults to None.
+                Yraw (np.ndarray) : (Optional) raw form of Y before preprocessing to remain associated with Y for visualization. Defaults to None. 
+
+            Returns:
+                dataset : the newly constructed Dataset object. (Dataset)
+        '''
+        
+        # check inputs
+        assert isinstance(X, np.ndarray), \
+            'X should be of type np.ndarray. Actual type: {}'.format(type(X))
+        assert isinstance(Y, np.ndarray), \
+            'Y should be of type np.ndarray. Actual type: {}'.format(type(Y))
+        assert isinstance(Xraw, (np.ndarray, type(None))), \
+            'Xraw should be of type np.ndarray or NoneType. ' + \
+                'Actual type: {}'.format(type(Xraw))
+        assert isinstance(Yraw, (np.ndarray, type(None))), \
+            'Yraw should be of type np.ndarray or NoneType. ' + \
+                'Actual type: {}'.format(type(Yraw))
+        assert isinstance(dataset_name, str), 'dataset_name should be of ' + \
+            'type str. Actual type: {}'.format(type(dataset_name))
+
+        # check that we haven't added a Dataset by this name already
+        assert dataset_name not in self.datasets.keys(), \
+            'A Dataset named {} has already been added to this Experiment.'.format(dataset_name)
+
+        # make new Dataset, add to Experiment's dict of datasets
+        dataset = Dataset(X=X, Y=Y, Xraw=Xraw, Yraw=Yraw, name=dataset_name)
+        self.datasets[dataset_name] = dataset
+        return dataset
+
+    def get_dataset(self, dataset_name):
+
+        ''' Retrieve a Dataset that has been registered with this Experiment.
+
+            Arguments:
+                dataset_name : name of the Dataset to retrieve. (str)
+
+            Returns: 
+                dataset : the Dataset associated with dataset_name. (Dataset) 
+        '''
+
+        # check inputs
+        assert isinstance(dataset_name, str), \
+            'dataset_name should be of type str.'
+        assert dataset_name in self.datasets.keys(), "No dataset with the " + \
+            "name 'dataset_name' has been added to this Experiment yet."
+
+        return self.datasets[dataset_name]
+
+    def get_data_info(self): 
+        return self.data_info
+
+    # TODO: this function doesn't work right now
+    def load_results_from_file(self, dataset_name='dataset_train'):
+        ''' Load and return saved results from running a given dataset
+            through the Experiment pipeline. This function differs from 
+            `retrieve_results()` because this loads the saved results 
+            from their save directory
+
+            Arguments: 
+                dataset_name (str) : name of Dataset to load results for. Defaults
+                               to the dataset used to train the pipeline, 
+                               'dataset_train'. 
+
+            Returns:
+                dict of dicts : dictionary of results-dictionaries. The first key
+                          specifies which Block the results come from. The
+                          second key specifies the specific result.
+        '''
+
+        # check inputs
+        assert isinstance(dataset_name, str), \
+            'dataset_name should be of type str.'
+
+        # pull corresponding Dataset from providied name
+        dataset = self.get_dataset(dataset_name)
+
+        # find directory for train_dataset results
+        dir_name = os.path.join(self.save_path, dataset.get_name())
+
+        # load in results for each block
+        results = {}
+        for block in self.blocks:
+            file_name = os.path.join(dir_name, block.get_name())
+            with open(file_name, 'rb') as f:
+                results[block.get_name()] = pickle.load(f)
+        
+        return results
+
+    def retrieve_results(self, dataset_name='dataset_train'): 
+        '''Returns the results from running a given dataset
+            through the Experiment pipeline. Default is the training dataset
+
+        Arguments: 
+            dataset_name (str) : name of Dataset to load results for. Defaults
+                            to the dataset used to train the pipeline, 
+                            'dataset_train'. 
+        Returns:
+            dict of dicts : dictionary of results-dictionaries. The first key
+                        specifies which Block the results come from. The
+                        second key specifies the specific result.
+        '''
+        dataset = self.datasets['dataset_train']
+        return dataset.cfl_results
+
+    # def check_blocks_compatibility(self):
+    #     ''' Check that each Block in self.blocks is compatible with the Blocks
+    #         that precede and follow it. 
+    #         TODO: This is currently not implemented and simply states that all
+    #         Blocks are always compatible. We may use class registration here
+    #         to determine block compatibility. Alternatively, we may have each
+    #         Block descendent specify it's argument expectations and return
+    #         guarantees. 
+
+    #         Arguments: None
+            
+    #         Returns:
+    #             compatible : whether or not self.blocks is a compatible
+    #                          composition of Blocks. (bool) 
+    #     '''
+
+    #     return True
+    def get_save_path(self):
+        ''' Return the path at which experiment results are saved.
+            Arguments: None
+            Returns:
+                str : path to experiment
+        '''
+        return self.save_path
+
+    def get_intervention_recs(self, dataset_name, k_samples=100, eps=0.5):
+        ''' For a given dataset, this function selects a subset of samples that
+            serve as optimal interventions to perform when testing for confounding
+            in CFL's observational partition.
+            TODO: complete documentation
+        '''
+        assert self.blocks is not None, 'blocks have not been defined yet.'
+        
+        cfl_results = self.get_dataset(dataset_name).get_cfl_results()
+        assert cfl_results is not None, 'There are no results for this Dataset.'
+
+        # TODO: standardize block names as CDE and Clusterer so those can be
+        # hardcoded here. Right now there's no way to know if blocks[0] is actually
+        # a CDE
+        return intervention_rec.get_recommendations(
+                    pyx=cfl_results[self.blocks[0].get_name()]['pyx'], 
+                    cluster_labels=cfl_results[self.blocks[1].get_name()]['x_lbls'], 
+                    k_samples=k_samples, 
+                    eps=eps,
+                    to_plot=False,
+                    series='series'
+                )    
+
+    def __propagate_verbosity(self, verbose, block_params):
+        for pi in range(len(block_params)):
+            if 'verbose' not in block_params[pi].keys():
+                modified_params = block_params[pi]
+                modified_params['verbose'] = verbose
+                block_params[pi] = modified_params
+        return block_params
+    
+    def __make_exp_dir(self, results_path):
+        ''' Build directory to which to save this Experiment. 
+            
+            Arguments: 
+                results_path (str): where to build this Experiment's directory. For
+                               example, if `results_path='path/to/dir'`, the
+                               direcotry `'path/to/dir/experiment000x'` will
+                               be built.
+            Returns: 
+                save_path (str): path to experiment directory 
+                            (`'path/to/dir/experiment000x'` in the example above).
+                            
+        '''
+
+        # check inputs
+        assert isinstance(results_path, str), \
+            'results_path should be of type str.'
+
+        # make sure base_path exists, if not make it
+        if not os.path.exists(results_path):
+            if self.verbose > 0:
+                print("save_path '{}' doesn't exist, creating now.".format(results_path))
+            os.makedirs(results_path)
+
+        # create dir for this run
+        save_path = os.path.join(results_path, get_next_dirname(results_path))
+        if self.verbose > 0:
+            print('All results from this run will be saved to {}'.format(save_path))
+        os.mkdir(save_path)
+
+        # make trained_blocks dir
+        os.mkdir(os.path.join(save_path, 'trained_blocks'))
+        return save_path
+
+    def __build_block(self, block_name, block_param):
+        ''' Given a Block's name and associated params, instantiate a Block 
+            object. 
+
+            Arguments: 
+                block_name : name of Block to instantiate. (str)
+                block_param : dictionary of parameters that specify this 
+                              Block. (dict)
+
+            Returns: instantiated Block. (Block descendent)
+
+            Note: for now, I will just implement this using the dict translation
+            method. Once we have time, we can look into using the registration
+            method.
+        '''
+
+        # check inputs
+        assert isinstance(block_name, str), 'block_name should be of type str.'
+        assert isinstance(block_param, dict), \
+            'block_param should be of type dict.'
+        assert block_name in BLOCK_KEY.keys(), block_name + ' has not been ' + \
+            'added to BLOCK_KEY yet. Please do so before proceeding. Note: ' + \
+            'this is a temporary system until we set up Block registration.' 
+
+        return BLOCK_KEY[block_name](data_info=self.data_info, 
+                                     params=block_param)
+
     def __save_results(self, results, dataset, block):
         ''' Save results for a given dataset and block. 
+            The Dataset's name (as specified when the Dataset object was
+            created) is saved to identify the source of the results. 
 
             Arguments: 
                 results (dict) : dictionary of results from running block on dataset. 
@@ -306,7 +546,6 @@ class Experiment():
             with open(fn, 'wb') as f:
                 pickle.dump(block_graph, f) 
     
-
     def __load_params(self, params_path):
         ''' Helper function to load params from a specified previous
             experiment to be used in this experiment. Primarily used in 
@@ -336,215 +575,6 @@ class Experiment():
                 block_params.append(pickle.load(f))
         return block_graph, block_params
 
-
-    def add_dataset(self, X, Y, dataset_name, Xraw=None, Yraw=None):
-        ''' Add a new dataset to be tracked by this Experiment. 
-            
-            Arguments: 
-                X (np.array) : X data of shape (n_samples, n_x_features) associated with 
-                    this Dataset. 
-                Y (np.array) : Y data of shape (n_samples, n_y_features) associated with
-                    this Dataset. 
-                dataset_name (str) : name associated with this Dataset. This will be
-                               the name used to retrieve a dataset using the
-                               `Experiment.get_dataset()` method. 
-                Xraw (np.ndarray) : (Optional) raw form of X before preprocessing to remain associated with X for visualization. Defaults to None.
-                Yraw (np.ndarray) : (Optional) raw form of Y before preprocessing to remain associated with Y for visualization. Defaults to None. 
-
-            Returns:
-                dataset : the newly constructed Dataset object. (Dataset)
-        '''
-        
-        # check inputs
-        assert isinstance(X, np.ndarray), \
-            'X should be of type np.ndarray. Actual type: {}'.format(type(X))
-        assert isinstance(Y, np.ndarray), \
-            'Y should be of type np.ndarray. Actual type: {}'.format(type(Y))
-        assert isinstance(Xraw, (np.ndarray, type(None))), \
-            'Xraw should be of type np.ndarray or NoneType. ' + \
-                'Actual type: {}'.format(type(Xraw))
-        assert isinstance(Yraw, (np.ndarray, type(None))), \
-            'Yraw should be of type np.ndarray or NoneType. ' + \
-                'Actual type: {}'.format(type(Yraw))
-        assert isinstance(dataset_name, str), 'dataset_name should be of ' + \
-            'type str. Actual type: {}'.format(type(dataset_name))
-
-        # check that we haven't added a Dataset by this name already
-        assert dataset_name not in self.datasets.keys(), \
-            'A Dataset named {} has already been added to this Experiment.'.format(dataset_name)
-
-        # make new Dataset, add to Experiment's dict of datasets
-        dataset = Dataset(X=X, Y=Y, Xraw=Xraw, Yraw=Yraw, name=dataset_name)
-        self.datasets[dataset_name] = dataset
-        return dataset
-
-    def get_dataset(self, dataset_name):
-        ''' Retrieve a Dataset that has been registered with this Experiment.
-
-            Arguments:
-                dataset_name : name of the Dataset to retrieve. (str)
-
-            Returns: 
-                dataset : the Dataset associated with dataset_name. (Dataset) 
-        '''
-
-        # check inputs
-        assert isinstance(dataset_name, str), \
-            'dataset_name should be of type str.'
-        assert dataset_name in self.datasets.keys(), "No dataset with the " + \
-            "name 'dataset_name' has been added to this Experiment yet."
-
-        return self.datasets[dataset_name]
-
-    def load_dataset_results(self, dataset_name='dataset_train'):
-        ''' Load and return all saved results from running a given dataset
-            through the Experiment pipeline.
-
-            Arguments: 
-                dataset_name (str) : name of Dataset to load results for. Defaults
-                               to the dataset used to train the pipeline, 
-                               'dataset_train'. 
-
-            Returns:
-                dict of dicts : dictionary of results-dictionaries. The first key
-                          specifies which Block the results come from. The
-                          second key specifies the specific result.
-        '''
-
-        # check inputs
-        assert isinstance(dataset_name, str), \
-            'dataset_name should be of type str.'
-
-        # pull corresponding Dataset from providied name
-        dataset = self.get_dataset(dataset_name)
-
-        # find directory for train_dataset results
-        dir_name = os.path.join(self.save_path, dataset.get_name())
-
-        # load in results for each block
-        results = {}
-        for block in self.blocks:
-            file_name = os.path.join(dir_name, block.get_name())
-            with open(file_name, 'rb') as f:
-                results[block.get_name()] = pickle.load(f)
-        
-        return results
-
-    
-    def __build_block(self, block_name, block_param):
-        ''' Given a Block's name and associated params, instantiate a Block 
-            object. 
-
-            Arguments: 
-                block_name : name of Block to instantiate. (str)
-                block_param : dictionary of parameters that specify this 
-                              Block. (dict)
-
-            Returns: instantiated Block. (Block descendent)
-
-            Note: for now, I will just implement this using the dict translation
-            method. Once we have time, we can look into using the registration
-            method.
-        '''
-
-        # check inputs
-        assert isinstance(block_name, str), 'block_name should be of type str.'
-        assert isinstance(block_param, dict), \
-            'block_param should be of type dict.'
-        assert block_name in BLOCK_KEY.keys(), block_name + ' has not been ' + \
-            'added to BLOCK_KEY yet. Please do so before proceeding. Note: ' + \
-            'this is a temporary system until we set up Block registration.' 
-
-        return BLOCK_KEY[block_name](data_info=self.data_info, 
-                                     params=block_param)
-
-
-    # def check_blocks_compatibility(self):
-    #     ''' Check that each Block in self.blocks is compatible with the Blocks
-    #         that precede and follow it. 
-    #         TODO: This is currently not implemented and simply states that all
-    #         Blocks are always compatible. We may use class registration here
-    #         to determine block compatibility. Alternatively, we may have each
-    #         Block descendent specify it's argument expectations and return
-    #         guarantees. 
-
-    #         Arguments: None
-            
-    #         Returns:
-    #             compatible : whether or not self.blocks is a compatible
-    #                          composition of Blocks. (bool) 
-    #     '''
-
-    #     return True
-
-    
-    def __make_exp_dir(self, results_path):
-        ''' Build directory to which to save this Experiment. 
-            
-            Arguments: 
-                results_path (str): where to build this Experiment's directory. For
-                               example, if `results_path='path/to/dir'`, the
-                               direcotry `'path/to/dir/experiment000x'` will
-                               be built.
-            Returns: 
-                save_path (str): path to experiment directory 
-                            (`'path/to/dir/experiment000x'` in the example above).
-                            
-        '''
-
-        # check inputs
-        assert isinstance(results_path, str), \
-            'results_path should be of type str.'
-
-        # make sure base_path exists, if not make it
-        if not os.path.exists(results_path):
-            if self.verbose > 0:
-                print("save_path '{}' doesn't exist, creating now.".format(results_path))
-            os.makedirs(results_path)
-
-        # create dir for this run
-        save_path = os.path.join(results_path, get_next_dirname(results_path))
-        if self.verbose > 0:
-            print('All results from this run will be saved to {}'.format(save_path))
-        os.mkdir(save_path)
-
-        # make trained_blocks dir
-        os.mkdir(os.path.join(save_path, 'trained_blocks'))
-        return save_path
-
-    def __propagate_verbosity(self, verbose, block_params):
-        for pi in range(len(block_params)):
-            if 'verbose' not in block_params[pi].keys():
-                modified_params = block_params[pi]
-                modified_params['verbose'] = verbose
-                block_params[pi] = modified_params
-        return block_params
-
-    def get_intervention_recs(self, dataset_name, k_samples=100, eps=0.5):
-        ''' For a given dataset, this function selects a subset of samples that
-            serve as optimal interventions to perform when testing for confounding
-            in CFL's observational partition.
-            TODO: complete documentation
-        '''
-        assert self.blocks is not None, 'blocks have not been defined yet.'
-        
-        cfl_results = self.get_dataset(dataset_name).get_cfl_results()
-        assert cfl_results is not None, 'There are no results for this Dataset.'
-
-        # TODO: standardize block names as CDE and Clusterer so those can be
-        # hardcoded here. Right now there's no way to know if blocks[0] is actually
-        # a CDE
-        return intervention_rec.get_recommendations(
-                    pyx=cfl_results[self.blocks[0].get_name()]['pyx'], 
-                    cluster_labels=cfl_results[self.blocks[1].get_name()]['x_lbls'], 
-                    k_samples=k_samples, 
-                    eps=eps,
-                    to_plot=False,
-                    series='series'
-                ) 
-        
-
-
 ########### HELPER FUNCTIONS ##################################################
 
 def get_next_dirname(path):
@@ -556,8 +586,6 @@ def get_next_dirname(path):
         next subdirectory name. 
     '''
     i = 0
-    while os.path.exists(os.path.join(path, 'experiment{}'.format(str(i).zfill(4)))):
+    while os.path.exists(os.path.join(path, 'experiment{}'.format(str(i).zfill(4)))): #zfill(4) creates a 4 digit number
         i += 1  
     return 'experiment{}'.format(str(i).zfill(4))
-
-
